/*
 * Tag menu
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "TagMenu.h"

#include "Environment.h"
#include "DebtagsDocument.h"

#include <apt-front/cache/component/debtags/utils.h>
#include <tagcoll/CardinalityStore.h>
#include <tagcoll/SmartHierarchy.h>

#include <map>
#include <vector>
#include <string>

using namespace std;
using namespace Tagcoll;
using namespace aptFront::cache::component::debtags;

//template class TagMenu<DebtagsDocument>;

template<class DOC>
void TagMenu<DOC>::do_selected(Tag tag)
{
	_signal_selected.emit(tag);
}

template<class DOC>
void TagMenu<DOC>::populateWith(const OpSet<Tag>& tags) throw ()
{
	// Needs std:: not to be confused with some other map in gtkmm
	std::map<Facet, vector<Tag> > contents;
	for (typename OpSet<Tag>::const_iterator i = tags.begin(); i != tags.end(); i++)
		contents[i->facet()].push_back(*i);

	for (typename std::map<Facet, vector<Tag> >::const_iterator i = contents.begin(); i != contents.end(); i++)
	{
		Gtk::Menu* submenu = manage(new Gtk::Menu);
		for (typename vector<Tag>::const_iterator j = i->second.begin(); j != i->second.end(); j++)
		{
			submenu->items().push_back(Gtk::Menu_Helpers::MenuElem(
						"[" + j->name() + "] " + j->shortDescription(),
						sigc::bind<Tag>(
							sigc::mem_fun(*this, &TagMenu::do_selected), *j)));
			//printf("Adding %.*s::%.*s\n", PFSTR(i->first), PFSTR(*j));
		}
		if (i->first.name().empty())
			items().push_back(Gtk::Menu_Helpers::MenuElem("(legacy)", *submenu));
		else
			items().push_back(Gtk::Menu_Helpers::MenuElem("[" + i->first.name() + "] " + i->first.shortDescription(), *submenu));
	}
}

template<class DOC>
void TagMenu<DOC>::populateWith(const OpSet<Tag>& tags, const Facet& facet) throw ()
{
	for (typename OpSet<Tag>::const_iterator i = tags.begin(); i != tags.end(); i++)
	{
		//printf("Facet: %.*s, tag: %.*s\n", PFSTR(f), PFSTR(t));
		if (i->facet() == facet)
		{
			//printf("Adding %.*s\n", PFSTR(*i));
			items().push_back(Gtk::Menu_Helpers::MenuElem(
						i->name(),
						sigc::bind<Tag>(
							sigc::mem_fun(*this, &TagMenu::do_selected), *i)));
		}
	}
}

template<class DOC>
void TagMenu<DOC>::populateWith(const OpSet<Tag>& tags, const OpSet<Facet>& toplevelFacets) throw ()
{
	// Needs std:: not to be confused with some other map
	std::map<Facet, vector<Tag> > contents;
    for (typename OpSet<Tag>::const_iterator i = tags.begin(); i != tags.end(); i++)
	{
		// Filter out non-toplevel facets
		if (!toplevelFacets.contains(i->facet()))
			continue;

		typename std::map<Facet, vector<Tag> >::iterator j = contents.find(i->facet());
		if (j != contents.end())
			j->second.push_back(*i);
		else
		{
			std::vector<Tag> v;
			v.push_back(*i);
			contents.insert(pair<Facet, vector<Tag> >(i->facet(), v));
		}
	}

	for (typename std::map<Facet, vector<Tag> >::const_iterator i = contents.begin(); i != contents.end(); i++)
	{
		Gtk::Menu* submenu = manage(new Gtk::Menu);
		for (typename vector<Tag>::const_iterator j = i->second.begin(); j != i->second.end(); j++)
		{
			submenu->items().push_back(Gtk::Menu_Helpers::MenuElem(
						j->name(),
						sigc::bind<Tag>(
							sigc::mem_fun(*this, &TagMenu::do_selected), *j)));
			//printf("Adding %.*s::%.*s\n", PFSTR(i->first), PFSTR(*j));
		}
		if (i->first.name().empty())
			items().push_back(Gtk::Menu_Helpers::MenuElem("(legacy)", *submenu));
		else
			items().push_back(Gtk::Menu_Helpers::MenuElem(i->first.name(), *submenu));
	}
}


template<class DOC>
void TagMenu<DOC>::populateAll(const DOC& doc) throw ()
{
	populateWith(doc.vocabulary().tags());
}

template<class DOC>
void TagMenu<DOC>::populateAvailable(const DOC& doc, const OpSet<Tag>& selectedTags, const Collection<Package, Tag>& items) throw ()
{
	/*
	OpSet<Tag> merged;
	OpSet<Tag> inters;

	// Create merged tagset
	// Create intersected tagset
	for (PackageSet::const_iterator i = items.begin(); i != items.end(); i++)
	{
		OpSet<Tag> tags = (*i).tags();
		if (i == items.begin())
			inters = tags;
		else
			inters ^= tags;
		merged += tags;
	}

	// Use merged - intersected
	populateWith(merged - inters);
	*/
	populateWith(items.getAllTags() - selectedTags);
}

template<class DOC>
void TagMenu<DOC>::populateAvailableSmart(
		const DOC& doc,
		const OpSet<Tag>& selectedTags,
		const Collection<Package, Tag>& items) throw ()
{
	// Compute the minimum set of toplevel facets that can be used to select
	// all items in `items'

	CardinalityStore<Package, Facet> coll;
	OpSet<Facet> toplevelFacets;

	// Make a new collection tagged with facets instead of tags
	TagToFacet<Package> tagStripper(coll);
	items.output(tagStripper);

	Facet f;
	SmartHierarchyNode<Package, Facet> node(f, coll, 0);
	for (typename HierarchyNode<Package, Facet>::iterator i = node.begin();
			i != node.end(); i++)
		toplevelFacets += (*i)->tag();

	if (toplevelFacets.size() > 5)
		populateWith(items.getAllTags() - selectedTags, toplevelFacets);
	else
		populateWith(items.getAllTags() - selectedTags);
}

template<class DOC>
void TagMenu<DOC>::populateAvailable(const DOC& doc, const OpSet<Tag>& selectedTags) throw ()
{
	OpSet<Tag> tags = selectedTags.empty() ?
		  doc.debtags().tagdb().getAllTags()
		: doc.debtags().tagdb().getCompanionTags(selectedTags);
	//OpSet<Tag> tags = Debtags::Environment::get().tagdb().getCompanionTags(selectedTags);
	populateWith(tags);
}

template<class DOC>
void TagMenu<DOC>::populateAvailable(const DOC& doc, const OpSet<Tag>& selectedTags, const Facet& facet) throw ()
{
	OpSet<Tag> tags = doc.debtags().tagdb().getCompanionTags(selectedTags);
	populateWith(tags, facet);
}

template<class DOC>
void TagMenu<DOC>::populateUnselected(const DOC& doc, const OpSet<Tag>& selectedTags) throw ()
{
	OpSet<Tag> tags = doc.vocabulary().tags() - selectedTags;
	//for (OpSet<string>::const_iterator i = tags.begin(); i != tags.end(); i++)
		//printf("Sel: %.*s\n", PFSTR(*i));
	populateWith(tags);
}

template<class DOC>
void TagMenu<DOC>::populateUnselected(const DOC& doc, const OpSet<Tag>& selectedTags, const Facet& facet) throw ()
{
	OpSet<Tag> tags = doc.vocabulary().tags() - selectedTags;
	//for (OpSet<string>::const_iterator i = tags.begin(); i != tags.end(); i++)
		//printf("Sel: %.*s\n", PFSTR(*i));
	populateWith(tags, facet);
}



template class TagMenu<DebtagsDocument>;


// vim:set ts=4 sw=4:
