/*
 * tagcolledit - GUI for editing tagged collections
 *
 * Copyright (C) 2003  Enrico Zini
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#define APPNAME PACKAGE_NAME
#else
#warning No config.h found: using fallback values
#define APPNAME __FILE__
#define PACKAGE_VERSION "unknown"
#endif

#include <tagcoll/Exception.h>
#include <tagcoll/stringf.h>

#include <apt-front/init.h>
#include <apt-front/cache/cache.h>

#include "Environment.h"
#include "CommandlineParser.h"

#include "DebtagsDocument.h"
#include "DebtagsEditor.h"

#include <gtkmm/main.h>

using namespace std;
using namespace Tagcoll;

int main(int argc, char* argv[])
{
	try {
		// Install the handler for unexpected exceptions
		InstallUnexpected installUnexpected;

		CommandlineParser opts(APPNAME, "[options]",
				"GUI editor for Debian package tags\n"
				"Edits the Debian Package Tags database present on the system\n"
				"saving the changes in ~/.debtags-edit\n");
		opts.add("version", 0, "version", "print the program version, then exit");
		opts.add("verbose", 0, "verbose", "enable verbose output");
		opts.add("debug", 0, "debug", "enable debugging output (including verbose output)");

		// Process the commandline
		if (!opts.parse(argc, argv))
		{
			opts.printHelp();
			return 1;
		}
		if (opts.get("help").defined())
		{
			opts.printHelp();
			return 0;
		}
		if (opts.get("version").defined())
		{
			printf("%s ver." PACKAGE_VERSION "\n", APPNAME);
			return 0;
		}
		if (opts.get("verbose").defined())
			Environment::get().verbose(true);
		if (opts.get("debug").defined())
			Environment::get().debug(true);

		if (!aptFront::cache::component::PackageTags::hasTagDatabase())
		{
			fprintf(stderr, "Debtags database has not been found.  Please run debtags update and try again\n");
			return 1;
		}

		aptFront::init();
		// Opens the package cache
		// 0 is a pointer to an optional OpProgress object for status feedback
		aptFront::cache::Global::get().open(aptFront::cache::Cache::OpenDefault
				| aptFront::cache::Cache::OpenReadOnly
				| aptFront::cache::Cache::OpenTags
				| aptFront::cache::Cache::OpenDebtagsEditable);


		//Debtags::Environment::init(true);
		//printf("State dir is %.*s\n", PFSTR(Debtags::TagDB<Debtags::Package>::statedir()));

		// Read the full collection
		DebtagsDocument document;

		// Check for the existance of a database
		/* it's now done by debtags' postinst anyway
		if (!document.hasTagDatabase())
			warning("The tag database has not yet been generated: you need to run \"debtags update\" (as root) before using Package Tags\n");
		*/

		//document.load();

		Gtk::Main app(&argc, &argv);
		
		DebtagsEditor<DebtagsDocument> debtagsEditor(document);

		/*
		Debtags::Tag tag = document.vocabulary().getTag("accessibility::TODO");
		Tagcoll::OpSet<DebtagsDocument::package> pkgs = document.tagdb().getItems(tag);
		fprintf(stderr, "%d items\n", pkgs.size());
		*/

		Gtk::Main::run(debtagsEditor);

		//if (document.changed())
		//{
			printf("Please want: autosaving on exit...\n");
			document.save();
		//}
		
		return 0;
	}
	catch (Exception& e)
	{
		fatal_error("%s: %.*s\n", e.type(), PFSTR(e.desc()));
		return 1;
	}
}

// vim:set ts=4 sw=4:
